/////////////////////////////////////////////////////////////////////////////
//	Copyright (C) 1998 Chris Cant, PHD Computer Consultants Ltd
/////////////////////////////////////////////////////////////////////////////

#include "hiimwdm.h"

#pragma code_seg("PAGE")	// start PAGE section

/////////////////////////////////////////////////////////////////////////////
//	Define a shared memory area for all devices

const ULONG SHARED_MEMORY_SIZE = 4;

UCHAR SharedMemory[SHARED_MEMORY_SIZE];

/////////////////////////////////////////////////////////////////////////////

NTSTATUS WdmCreate(	IN PDEVICE_OBJECT fdo,
					IN PIRP Irp)
{
	// Complete successfully
	Irp->IoStatus.Status = STATUS_SUCCESS;
	Irp->IoStatus.Information = 0;
	IoCompleteRequest(Irp,IO_NO_INCREMENT);
	return STATUS_SUCCESS;
}

/////////////////////////////////////////////////////////////////////////////

NTSTATUS WdmClose(	IN PDEVICE_OBJECT fdo,
					IN PIRP Irp)
{
	// Complete successfully
	Irp->IoStatus.Status = STATUS_SUCCESS;
	Irp->IoStatus.Information = 0;
	IoCompleteRequest(Irp,IO_NO_INCREMENT);
	return STATUS_SUCCESS;
}
 
/////////////////////////////////////////////////////////////////////////////

NTSTATUS WdmRead(IN PDEVICE_OBJECT fdo,
				IN PIRP Irp)
{
	PWDM_DEVICE_EXTENSION dx = (PWDM_DEVICE_EXTENSION)fdo->DeviceExtension;
	PIO_STACK_LOCATION pIrpStack = IoGetCurrentIrpStackLocation(Irp);
	NTSTATUS status = STATUS_SUCCESS;
	ULONG BytesTxd = 0;

	ULONG ReadLen = pIrpStack->Parameters.Read.Length;

	if( ReadLen>SHARED_MEMORY_SIZE)
		status = STATUS_INVALID_PARAMETER;
	else if( ReadLen>0)
	{
		RtlMoveMemory( Irp->AssociatedIrp.SystemBuffer, SharedMemory, ReadLen);
		BytesTxd = ReadLen;
	}

	// Complete IRP
	Irp->IoStatus.Status = status;
	Irp->IoStatus.Information = BytesTxd;
	IoCompleteRequest(Irp,IO_NO_INCREMENT);
	return status;
}

/////////////////////////////////////////////////////////////////////////////

NTSTATUS WdmWrite(	IN PDEVICE_OBJECT fdo,
					IN PIRP Irp)
{
	PWDM_DEVICE_EXTENSION dx = (PWDM_DEVICE_EXTENSION)fdo->DeviceExtension;
	PIO_STACK_LOCATION	pIrpStack = IoGetCurrentIrpStackLocation(Irp);
	NTSTATUS status = STATUS_SUCCESS;
	ULONG BytesTxd = 0;

	ULONG WriteLen = pIrpStack->Parameters.Write.Length;

	if( WriteLen>SHARED_MEMORY_SIZE)
		status = STATUS_INVALID_PARAMETER;
	else if( WriteLen>0)
	{
		RtlMoveMemory( SharedMemory, Irp->AssociatedIrp.SystemBuffer, WriteLen);
		BytesTxd = WriteLen;
	}

	// Complete IRP
	Irp->IoStatus.Status = status;
	Irp->IoStatus.Information = BytesTxd;
	IoCompleteRequest(Irp,IO_NO_INCREMENT);
	return status;
}

/////////////////////////////////////////////////////////////////////////////

NTSTATUS WdmDeviceControl(	IN PDEVICE_OBJECT fdo,
							IN PIRP Irp)
{
	// Complete successfully
	Irp->IoStatus.Status = STATUS_SUCCESS;
	Irp->IoStatus.Information = 0;
	IoCompleteRequest(Irp,IO_NO_INCREMENT);
	return STATUS_SUCCESS;
}

#pragma code_seg()	// end PAGE section
