//////////////////////////////////////////////////////////////////////////////
//	Copyright (C) 1998 Chris Cant, PHD Computer Consultants Ltd
/////////////////////////////////////////////////////////////////////////////
//	1.01	14-Dec-98	CC	HiImWdmRegistryPath not in INI section
/////////////////////////////////////////////////////////////////////////////

#include "hiimwdm.h"

/////////////////////////////////////////////////////////////////////////////

UNICODE_STRING HiImWdmRegistryPath;

#pragma code_seg("INIT") // start PAGE section

/////////////////////////////////////////////////////////////////////////////
//	DriverEntry:
//
//	Description:
//		This function initializes the driver, and creates
//		any objects needed to process I/O requests.
//
//	Arguments:
//		Pointer to the Driver object
//		Registry path string for driver service key
//
//	Return Value:
//		This function returns STATUS_XXX

extern "C"
NTSTATUS DriverEntry(	IN PDRIVER_OBJECT DriverObject,
						IN PUNICODE_STRING RegistryPath)
{
	// Save a copy of our RegistryPath for WMI
	HiImWdmRegistryPath.MaximumLength = RegistryPath->MaximumLength;
	HiImWdmRegistryPath.Length = RegistryPath->Length;
	HiImWdmRegistryPath.Buffer = (PWSTR)ExAllocatePool(PagedPool, HiImWdmRegistryPath.MaximumLength);
	if( HiImWdmRegistryPath.Buffer == NULL) return STATUS_INSUFFICIENT_RESOURCES;

	RtlZeroMemory( HiImWdmRegistryPath.Buffer, HiImWdmRegistryPath.MaximumLength);
	RtlMoveMemory( HiImWdmRegistryPath.Buffer, RegistryPath->Buffer, RegistryPath->Length);
	
	// Export other driver entry points...
 	DriverObject->DriverExtension->AddDevice = WdmAddDevice;
 	DriverObject->DriverUnload = WdmUnload;

	DriverObject->MajorFunction[IRP_MJ_CREATE] = WdmCreate;
	DriverObject->MajorFunction[IRP_MJ_CLOSE] = WdmClose;

	DriverObject->MajorFunction[IRP_MJ_PNP] = WdmPnp;
	DriverObject->MajorFunction[IRP_MJ_POWER] = WdmPower;

	DriverObject->MajorFunction[IRP_MJ_READ] = WdmRead;
	DriverObject->MajorFunction[IRP_MJ_WRITE] = WdmWrite;
	DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL] = WdmDeviceControl;

	DriverObject->MajorFunction[IRP_MJ_SYSTEM_CONTROL] = WdmSystemControl;

	return STATUS_SUCCESS;
}
#pragma code_seg() // end INIT section

//////////////////////////////////////////////////////////////////////////////
//	AbcUnload
//
//	Description:
//		Unload the driver by removing any remaining objects, etc.
//
//	Arguments:
//		Pointer to the Driver object
//
//	Return Value:
//		None

#pragma code_seg("PAGE") // start PAGE section

VOID WdmUnload(IN PDRIVER_OBJECT DriverObject)
{
	ExFreePool(HiImWdmRegistryPath.Buffer);
}

//////////////////////////////////////////////////////////////////////////////
#pragma code_seg() // end PAGE section
