/////////////////////////////////////////////////////////////////////////////
//	Copyright (C) 1998 Chris Cant, PHD Computer Consultants Ltd
/////////////////////////////////////////////////////////////////////////////

#define INITGUID // initialize WDM_GUID in this module

#include "hiimwdm.h"

#pragma code_seg("PAGE")	// start PAGE section

/////////////////////////////////////////////////////////////////////////////
//	WdmAddDevice:
//
//	Description:
//		Cope with a new Pnp device being added here.
//		Usually just attach to the top of the driver stack.
//		Do not talk to device here!
//
//	Arguments:
//		Pointer to the Driver object
//		Pointer to Physical Device Object
//
//	Return Value:
//		This function returns STATUS_XXX

NTSTATUS WdmAddDevice(	IN PDRIVER_OBJECT DriverObject,
						IN PDEVICE_OBJECT pdo)
{
	NTSTATUS status;
    PDEVICE_OBJECT fdo;

	// Create our Functional Device Object in fdo
	status = IoCreateDevice(DriverObject,
		sizeof(WDM_DEVICE_EXTENSION),
		NULL,	// No Name
		FILE_DEVICE_UNKNOWN,
		0,
		FALSE,	// Not exclusive
		&fdo);
	if( NT_ERROR(status))
		return status;

	// Remember fdo in our device extension
	PWDM_DEVICE_EXTENSION dx = (PWDM_DEVICE_EXTENSION)fdo->DeviceExtension;
	dx->pdo = pdo;
	dx->fdo = fdo;

	// Register and enable our device interface
	status = IoRegisterDeviceInterface(pdo, &WDM_GUID, NULL, &dx->ifSymLinkName);
	if( NT_ERROR(status))
	{
		IoDeleteDevice(fdo);
		return status;
	}
	IoSetDeviceInterfaceState(&dx->ifSymLinkName, TRUE);

	// Attach to the driver stack below us
	dx->NextDevice = IoAttachDeviceToDeviceStack(fdo,pdo);

	// Set fdo flags appropriately
	fdo->Flags &= ~DO_DEVICE_INITIALIZING;
	fdo->Flags |= DO_BUFFERED_IO;

	RegisterWmi(fdo);

	return STATUS_SUCCESS;
}

/////////////////////////////////////////////////////////////////////////////
//	WdmPnp:
//
//	Description:
//		Handle IRP_MJ_PNP requests
//
//	Arguments:
//		Pointer to the Device object
//		Pointer to the IRP
//
//	Return Value:
//		This function returns STATUS_XXX

NTSTATUS WdmPnp(	IN PDEVICE_OBJECT fdo,
					IN PIRP Irp)
{
	PWDM_DEVICE_EXTENSION dx=(PWDM_DEVICE_EXTENSION)fdo->DeviceExtension;

	// Remember minor function
	PIO_STACK_LOCATION stack = IoGetCurrentIrpStackLocation(Irp);
	ULONG MinorFunction = stack->MinorFunction;

	// Just pass to lower driver
	IoSkipCurrentIrpStackLocation(Irp);
	NTSTATUS status = IoCallDriver( dx->NextDevice, Irp);

	// Device removed
	if( MinorFunction==IRP_MN_REMOVE_DEVICE)
	{
		// disable device interface
		IoSetDeviceInterfaceState(&dx->ifSymLinkName, FALSE);
		RtlFreeUnicodeString(&dx->ifSymLinkName);

		DeregisterWmi(fdo);

		// unattach from stack
		if (dx->NextDevice)
			IoDetachDevice(dx->NextDevice);

		// delete our fdo
		IoDeleteDevice(fdo);
	}

	return status;
}

/////////////////////////////////////////////////////////////////////////////
//	WdmPower:
//
//	Description:
//		Handle IRP_MJ_POWER requests
//
//	Arguments:
//		Pointer to the Device object
//		Pointer to the IRP
//
//	Return Value:
//		This function returns STATUS_XXX

NTSTATUS WdmPower(	IN PDEVICE_OBJECT fdo,
					IN PIRP Irp)
{
	PWDM_DEVICE_EXTENSION dx = (PWDM_DEVICE_EXTENSION)fdo->DeviceExtension;

	// Just pass to lower driver
	PoStartNextPowerIrp( Irp);
	IoSkipCurrentIrpStackLocation(Irp);
	return PoCallDriver( dx->NextDevice, Irp);
}

#pragma code_seg()	// end PAGE section
